#![allow(non_snake_case)]
#[macro_use]
extern crate criterion;

extern crate games;

criterion_group!(
    BlackJack,
    black_jack::new_game,
    black_jack::freeze,
    black_jack::dethaw,
    black_jack::hit_n_finish
);
criterion_group!(CoinToss, coin_toss::new_game);
criterion_group!(RockPaperScissors, rps_game::new_game);
criterion_group!(SlotMachine, slot_machine::slots);
criterion_group!(Solitaire, solitaire::new_game);
criterion_main!(
    BlackJack,
    CoinToss,
    RockPaperScissors,
    SlotMachine,
    Solitaire
);

mod black_jack {
    use criterion::Criterion;
    use games::blackjack::*;
    pub fn new_game(c: &mut Criterion) {
        c.bench_function("New Game (Default)", |b| b.iter(|| BlackJack::new(None)));
    }
    pub fn freeze(c: &mut Criterion) {
        c.bench_function("Freeze New Game clone", |b| {
            let bj = BlackJack::new(None);
            b.iter(|| bj.clone().freeze())
        });
    }
    pub fn dethaw(c: &mut Criterion) {
        c.bench_function("Dethaw New Game", |b| {
            let fbj = BlackJack::new(None).freeze();
            b.iter(move || BlackJack::dethaw(fbj.clone()))
        });
    }
    pub fn hit_n_finish(c: &mut Criterion) {
        c.bench_function("Hit & Finish", |b| {
            let bj = BlackJack::new(None);
            b.iter(move || {
                let mut bj = bj.clone();
                bj.player_hit().is_ok();
                bj.finish();
            })
        });
    }
}
mod coin_toss {
    use criterion::Criterion;
    use games::coin_toss::*;
    pub fn new_game(c: &mut Criterion) {
        c.bench_function_over_inputs(
            "Coin Toss",
            |b, &guess| b.iter(|| CoinToss::guess(guess).expect("Shouldn't fail")),
            &["h", "t"],
        );
    }
}
mod rps_game {
    use criterion::Criterion;
    use games::rps::*;
    pub fn new_game(c: &mut Criterion) {
        c.bench_function_over_inputs(
            "Rock Paper Scissors",
            |b, &wep| b.iter(|| RpsGame::new(wep).expect("Shouldn't Fail").result()),
            &["Rock", "Paper", "Scissors"],
        );
    }
}
mod slot_machine {
    use criterion::Criterion;
    use games::slot_machine::SlotMachine;
    pub fn slots(c: &mut Criterion) {
        c.bench_function("Slot Machine", |b| b.iter(SlotMachine::new));
    }
}

mod solitaire {
    use criterion::Criterion;
    use games::solitaire::Solitaire;
    pub fn new_game(c: &mut Criterion) {
        c.bench_function("New Solitaire Game", |b| b.iter(Solitaire::new));
    }
}
