use crate::cards::StandardCard as Card;

/// Used to manage BlackJack hands
#[derive(Clone, Debug, Default)]
// #[derive(Clone, Debug)]
pub(crate) struct Hand {
    pub(crate) cards: Vec<Card>,
    // pub(crate) cards: [super::BlackJackCard;7]
}

impl Hand {
    /// Add a card to hand
    pub(crate) fn add_card(&mut self, card: Card) {
        self.cards.push(card)
    }

    /// Calculate the score
    pub(crate) fn score(&self) -> u8 {
        let (mut ace_count, mut total) = self.cards.iter().fold((0u8, 0u8), |mut d, c| {
            // Get the face value of the card
            let val = c.face().value();
            // If the value is 11, its an ace
            if val == 11 {
                // Increase the ace count by 1
                d.0 += 1;
            // Not an ace
            } else {
                // Increase the total by the face value
                d.1 += val;
            }
            d
        });
        // If there is more than one ace
        if ace_count >= 1 {
            // And the total is less than 10
            if total <= 10 {
                // The ace is worth 11 points
                total += 11;
            } else {
                // Otherwise the ace is worth 1
                total += 1;
            }
            // Reduce the ace count by 1
            ace_count -= 1;
        }
        // ace_count is to total of all other aces
        // because the value of the other aces is 1

        // Question: If the sum of aces + the 10 ace push it over 21, should the origional ace be set to the value of 1?
        total + ace_count
    }
}
