use super::{Weapon, WeaponParseError};
use std::str::FromStr;

/// Result of `RpsGame`
#[derive(Copy, Clone, Debug)]
pub enum GameResult {
    /// Player has won the game
    Win {
        /// The weapon the player chose
        player: Weapon,
        /// The weapon the robot chose
        robo: Weapon,
    },
    /// Player has lost the game
    Lose {
        /// The weapon the player chose
        player: Weapon,
        /// The weapon the robot chose
        robo: Weapon,
    },
    /// It is a draw
    Draw {
        /// The weapon the player chose
        player: Weapon,
        /// The weapon the robot chose
        robo: Weapon,
    },
}

/// Rock Paper scissors Game
#[derive(Copy, Clone, Debug)]
pub struct RpsGame {
    // Player's chosen weapon
    player: Weapon,
    // Bot's chosen Weapon
    robo: Weapon,
}

impl RpsGame {
    /// Create a new game of rock paper scissors
    /// weapon: Weapon you chose to use (Rock/Paper/Scissors)
    #[allow(clippy::new_ret_no_self)]
    pub fn new(weapon: &str) -> Result<Self, WeaponParseError> {
        Ok(Self {
            player: Weapon::from_str(weapon)?,
            robo: Weapon::rand(),
        })
    }
    /// convert the struct into a result of the game
    pub fn result(self) -> GameResult {
        self.into()
    }
}

impl std::convert::Into<GameResult> for RpsGame {
    fn into(self) -> GameResult {
        if self.player > self.robo {
            GameResult::Win {
                player: self.player,
                robo: self.robo,
            }
        } else if self.player == self.robo {
            GameResult::Draw {
                player: self.player,
                robo: self.robo,
            }
        } else {
            GameResult::Lose {
                player: self.player,
                robo: self.robo,
            }
        }
    }
}
